/*
 Copyright 2004-2008 Paul R. Holser, Jr.  All rights reserved.
 Licensed under the Academic Free License version 3.0
 */

package jaggregate;

/**
 * Utility class containing methods that work with arrays.
 *
 * @author <a href="mailto:pholser@alumni.rice.edu">Paul Holser</a>
 * @version $Id: Arrays.java,v 1.3 2008/03/30 03:07:10 pholser Exp $
 */
public class Arrays {
    /**
     * Discourages instantiation.
     *
     * @throws UnsupportedOperationException always
     */
    protected Arrays() {
        throw new UnsupportedOperationException();
    }

    /**
     * Creates and answers an array containing the given items.
     * <p/>
     * This is a more expressive, less repetitive way to create arrays.  For example:
     * <pre><code>
     *   Integer[] littlePrimes = arrayWith( 2, 3, 5, 7 );
     * </code></pre>
     * versus
     * <pre><code>
     *   Integer[] littlePrimes = new Integer[] { 2, 3, 5, 7 };
     * </code></pre>
     *
     * @param <E> the type of the items
     * @param items a variable-length list of items
     * @return the items as an array
     * @throws NullPointerException if the varargs list is {@code null}
     * @throws IllegalArgumentException if the varargs list is of length zero
     * @see <a href="http://www.thenewsbeforethenews.com/2006/05/02/java-5-syntax-tricks-to-keep-you-dry/">
     * Java 5 syntax tricks to keep you DRY</a>
     */
    public static <E> E[] arrayWith( E... items ) {
        if ( items.length == 0 )
            throw new IllegalArgumentException( "illegal zero-length varargs" );

        return items;
    }
}
