/*
 Copyright 2004-2008 Paul R. Holser, Jr.  All rights reserved.
 Licensed under the Academic Free License version 3.0
 */

package jaggregate;

/**
 * Interface for a predicate that accepts two arguments.
 * <p/>
 * This interface was extracted from {@link BinaryPredicate} and methods
 * using it retrofitted to use this interface, to allow BGGA-style closures to be
 * converted to an interface type representing the predicate.
 *
 * @param <A1> a constraint on the allowable types for the predicate's first argument
 * @param <A2> a constraint on the allowable types for the predicate's second argument
 * @author <a href="mailto:pholser@alumni.rice.edu">Paul Holser</a>
 * @version $Id: BinaryCondition.java,v 1.3 2008/10/03 19:01:23 pholser Exp $
 * @since 3.3
 */
public interface BinaryCondition<A1, A2> {
    /**
     * Evaluates this predicate against the given targets.
     *
     * @param first first argument to the predicate
     * @param second second argument to the predicate
     * @return the result of the evaluation
     */
    boolean matches( A1 first, A2 second );
}
