/*
 Copyright 2004-2008 Paul R. Holser, Jr.  All rights reserved.
 Licensed under the Academic Free License version 3.0
 */

package jaggregate;

/**
 * Interface for a functor that accepts two arguments.
 * <p/>
 * This interface was extracted from {@link BinaryFunction} and methods
 * using it retrofitted to use this interface, to allow BGGA-style closures to be
 * converted to an interface type representing the functor.
 *
 * @param <A1> a constraint on the allowable types for the functor's first argument
 * @param <A2> a constraint on the allowable types for the functor's second argument
 * @param <R> a constraint on the allowable types for the functor's return value
 * @author <a href="mailto:pholser@alumni.rice.edu">Paul Holser</a>
 * @version $Id: BinaryFunctor.java,v 1.3 2008/10/03 19:01:23 pholser Exp $
 * @since 3.3
 */
public interface BinaryFunctor<A1, A2, R> {
    /**
     * Evaluates this functor against the given targets.
     *
     * @param first first argument to the functor
     * @param second second argument to the functor
     * @return the result of the evaluation
     */
    R evaluate( A1 first, A2 second );
}
