/*
 Copyright 2004-2008 Paul R. Holser, Jr.  All rights reserved.
 Licensed under the Academic Free License version 3.0
 */

package jaggregate;

/**
 * A simple string-to-string pair, created by parsing a string of the form
 * {@code "key=value"}.
 *
 * @author <a href="mailto:pholser@alumni.rice.edu">Paul Holser</a>
 * @version $Id: KeyValuePair.java,v 1.2 2008/03/26 18:00:33 pholser Exp $
 */
public final class KeyValuePair extends Pair<String, String> {
    private static final long serialVersionUID = -1L;
    private static final String ZERO_LENGTH = "";

    private KeyValuePair( String key, String value ) {
        super( key, value );
    }

    /**
     * Creates a string-to-string pair by parsing a string of the form {@code
     * "key=value"}.
     *
     * @param aString the string representation of the pair
     * @return the pair
     */
    public static KeyValuePair parse( String aString ) {
        int indexOfEqualsSign = aString.indexOf( "=" );
        if ( indexOfEqualsSign == -1 )
            return new KeyValuePair( aString, ZERO_LENGTH );

        String parsedKey = aString.substring( 0, indexOfEqualsSign );
        String parsedValue;

        if ( indexOfEqualsSign == aString.length() - 1 )
            parsedValue = ZERO_LENGTH;
        else
            parsedValue = aString.substring( indexOfEqualsSign + 1 );

        return new KeyValuePair( parsedKey, parsedValue );
    }
}
