/*
 Copyright 2004-2008 Paul R. Holser, Jr.  All rights reserved.
 Licensed under the Academic Free License version 3.0
 */

package jaggregate;

/**
 * Provides protocol for a stream that allows objects to be added to its <dfn>past
 * sequence values</dfn>.
 *
 * @param <E> the type of elements in the stream
 * @author <a href="mailto:pholser@alumni.rice.edu">Paul Holser</a>
 * @version $Id: PuttableStream.java,v 1.3 2008/05/05 00:57:13 pholser Exp $
 */
public interface PuttableStream<E> {
    /**
     * Writes the given new element to this stream.
     * <p/>
     * Appends {@code newElement} to the past sequence values.  If the future sequence
     * values is not empty, removes its first object.
     *
     * @param newElement the element to add
     * @throws java.util.NoSuchElementException if there is no next element to replace
     */
    void nextPut( E newElement );

    /**
     * Enumerates the given collection, adding each new element to this stream.
     * <p/>
     * This message has the effect of enumerating {@code newElements} with
     * {@link Collection#forEachDo(UnaryFunctor) forEachDo} and adding each element to
     * this stream with {@link #nextPut(Object) nextPut}.
     *
     * @param newElements the elements to add
     * @throws NullPointerException if {@code newElements} is {@code null}
     * @throws java.util.NoSuchElementException if the put overruns the number of
     * elements in the stream
     */
    void nextPutAll( Collection<? extends E> newElements );
}
