/*
 Copyright 2004-2008 Paul R. Holser, Jr.  All rights reserved.
 Licensed under the Academic Free License version 3.0
 */

package jaggregate;

/**
 * Interface for a predicate that accepts one argument.
 * <p/>
 * This interface was extracted from {@link UnaryPredicate} and methods
 * using it retrofitted to use this interface, to allow BGGA-style closures to be
 * converted to an interface type representing the predicate.
 *
 * @param <A> a constraint on the allowable types for the predicate's argument.
 * @author <a href="mailto:pholser@alumni.rice.edu">Paul Holser</a>
 * @version $Id: UnaryCondition.java,v 1.3 2008/05/12 00:23:14 pholser Exp $
 * @since 3.3
 */
public interface UnaryCondition<A> {
    /**
     * Evaluates this predicate against the given target.
     *
     * @param target the argument to the predicate
     * @return the result of the evaluation
     */
    boolean matches( A target );
}
