/*
 Copyright 2004-2008 Paul R. Holser, Jr.  All rights reserved.
 Licensed under the Academic Free License version 3.0
 */

package jaggregate;

/**
 * Interface for a functor that accepts one argument.
 * <p/>
 * This interface was extracted from {@link UnaryFunctor} and methods using it
 * retrofitted to use this interface, to allow BGGA-style closures to be converted to an
 * interface type representing the functor.
 *
 * @param <A> a constraint on the allowable types for the functor's argument
 * @param <R> a constraint on the allowable types for the functor's return value
 * @author <a href="mailto:pholser@alumni.rice.edu">Paul Holser</a>
 * @version $Id: UnaryFunctor.java,v 1.3 2008/05/12 00:23:14 pholser Exp $
 * @since 3.3
 */
public interface UnaryFunctor<A, R> {
    /**
     * Evaluates this functor against the given target.
     *
     * @param argument the argument to the functor
     * @return the result of the evaluation
     */
    R evaluate( A argument );
}
