/*
 Copyright 2004-2008 Paul R. Holser, Jr.  All rights reserved.
 Licensed under the Academic Free License version 3.0
 */

package jaggregate.internal;

import static jaggregate.internal.CompilerWarnings.*;

/**
 * Facilities used internally for casting.
 *
 * @author <a href="mailto:pholser@alumni.rice.edu">Paul Holser</a>
 * @version $Id: Casting.java,v 1.2 2008/03/30 03:07:11 pholser Exp $
 */
public class Casting {
    /**
     * Discourages instantiation.
     *
     * @throws UnsupportedOperationException always
     */
    protected Casting() {
        throw new UnsupportedOperationException();
    }

    /**
     * Performs an unchecked cast of a given object to a given type.
     * <p/>
     * This method helps to eliminate compiler warnings from within Jaggregate code in
     * areas where "unsafe" casts are known to be safe yet cannot be suppressed easily,
     * at all, or without annotating at too broad of a scope or too frequently via
     * {@link SuppressWarnings}.  <em>Be careful</em>--don't use this method to try to
     * dodge responsibility for type-safe code.
     *
     * @see Class#cast(Object)
     * @param <T> a restriction on the type of the new reference to the object
     * @param item the object to cast
     * @return a reference to {@code item}, of a different type
     * @throws ClassCastException if the cast fails; that is, {@code item} cannot be
     * referenced by a variable of the chosen or inferred type
     */
    @SuppressWarnings( UNCHECKED )
    public static <T> T cast( Object item ) {
        return (T) item;
    }
}
