/*
 Copyright 2004-2008 Paul R. Holser, Jr.  All rights reserved.
 Licensed under the Academic Free License version 3.0
 */

package jaggregate.internal;

import static java.lang.System.*;

import jaggregate.Objects;
import static jaggregate.Objects.*;

/**
 * Describes a strategy for deciding whether two objects are equivalent.
 *
 * @author <a href="mailto:pholser@alumni.rice.edu">Paul Holser</a>
 * @version $Id: EquivalenceTester.java,v 1.1 2008/03/26 18:01:00 pholser Exp $
 */
public enum EquivalenceTester {
    /**
     * An equivalence tester that decides based on {@link Object#equals(Object) equals}.
     */
    OBJECT_ATTRIBUTES {
        /**
         * {@inheritDoc}
         */
        @Override
        public boolean areEqual( Object first, Object second ) {
            return Objects.areEqual( first, second );
        }

        /**
         * {@inheritDoc}
         */
        @Override
        public int hashCode( Object target ) {
            return nullSafeHashCode( target );
        }
    },

    /**
     * An equivalence tester that decides based on {@code ==}.
     */
    OBJECT_IDENTITIES {
        /**
         * {@inheritDoc}
         */
        @Override
        public boolean areEqual( Object first, Object second ) {
            return first == second;
        }

        /**
         * {@inheritDoc}
         */
        @Override
        public int hashCode( Object target ) {
            return identityHashCode( target );
        }
    };

    /**
     * Tells whether the two given objects are considered equivalent.
     *
     * @param first first comparand
     * @param second second comparand
     * @return whether {@code first} and {@code second} are considered equivalent
     */
    public abstract boolean areEqual( Object first, Object second );

    /**
     * Computes a hash code for the given object.
     *
     * @param target object to compute a hash for
     * @return the computed hash
     */
    public abstract int hashCode( Object target );
}
