/*
 Copyright 2004-2008 Paul R. Holser, Jr.  All rights reserved.
 Licensed under the Academic Free License version 3.0
 */

package jaggregate.internal;

import jaggregate.UnaryCondition;
import static jaggregate.internal.ArgumentChecks.*;

/**
 * @author <a href="mailto:pholser@alumni.rice.edu">Paul Holser</a>
 * @version $Id: UnaryConditions.java,v 1.2 2008/10/03 19:01:23 pholser Exp $
 */
public class UnaryConditions {
    /**
     * Discourages instantiation.
     *
     * @throws UnsupportedOperationException always
     */
    protected UnaryConditions() {
        throw new UnsupportedOperationException();
    }

    /**
     * Answers a predicate that represents the logical <dfn>inverse</dfn> of the given
     * predicate; wherever the given predicate's {@link UnaryCondition#matches(Object)
     * matches} method would answer {@code true}, the inverse answers {@code false};
     * and vice versa.
     *
     * @param <T> constraint on the types accepted by the condition
     * @param condition the condition to invert
     * @return the inverse of {@code condition}
     * @throws NullPointerException if {@code condition} is {@code null}
     */
    public static <T> UnaryCondition<T> not( final UnaryCondition<T> condition ) {
        ensureNotNull( condition, DISCRIMINATOR );

        return new UnaryCondition<T>() {
            public boolean matches( T target ) {
                return !condition.matches( target );
            }
        };
    }
}
