/*
 Copyright 2004-2008 Paul R. Holser, Jr.  All rights reserved.
 Licensed under the Academic Free License version 3.0
 */

package jaggregate.internal.number;

import java.math.BigDecimal;

/**
 * Wrapper for a {@code BigDecimal}.
 *
 * @author <a href="mailto:pholser@alumni.rice.edu">Paul Holser</a>
 * @version $Id: OperableBigDecimal.java,v 1.1 2008/03/26 18:01:28 pholser Exp $
 */
public class OperableBigDecimal extends OperableNumber<BigDecimal> {
    private static final long serialVersionUID = -1L;

    private static final OperableBigDecimal ONE =
        new OperableBigDecimal( BigDecimal.ONE ) {
            private static final long serialVersionUID = -1L;

            @Override
            public void incrementBy( OperableNumber<BigDecimal> increment ) {
                throw new UnsupportedOperationException();
            }
        };

    /**
     * Creates a wrapper for the given number.
     *
     * @param wrapped the wrapped number
     */
    public OperableBigDecimal( BigDecimal wrapped ) {
        super( wrapped );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean isZero() {
        return unwrapped().compareTo( BigDecimal.ZERO ) == 0;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableBigDecimal plus( BigDecimal addend ) {
        return new OperableBigDecimal( unwrapped().add( addend ) );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableBigDecimal plus( int addend ) {
        return new OperableBigDecimal(
            unwrapped().add( BigDecimal.valueOf( addend ) ) );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableBigDecimal minus( BigDecimal subtrahend ) {
        return new OperableBigDecimal( unwrapped().subtract( subtrahend ) );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableBigDecimal times( BigDecimal multiplier ) {
        return new OperableBigDecimal( unwrapped().multiply( multiplier ) );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableBigDecimal times( int multiplier ) {
        return new OperableBigDecimal(
            unwrapped().multiply( BigDecimal.valueOf( multiplier ) ) );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableBigDecimal dividedBy( BigDecimal divisor ) {
        return new OperableBigDecimal( unwrapped().divideToIntegralValue( divisor ) );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableBigDecimal modulo( BigDecimal divisor ) {
        return new OperableBigDecimal( unwrapped().remainder( divisor ) );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean isPositive() {
        return unwrapped().signum() > 0;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean isNegative() {
        return unwrapped().signum() < 0;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean greaterThan( BigDecimal comparand ) {
        return unwrapped().compareTo( comparand ) > 0;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean greaterThanOrEqualTo( BigDecimal comparand ) {
        return unwrapped().compareTo( comparand ) >= 0;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean lessThan( BigDecimal comparand ) {
        return unwrapped().compareTo( comparand ) < 0;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean lessThanOrEqualTo( BigDecimal comparand ) {
        return unwrapped().compareTo( comparand ) <= 0;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableBigDecimal negated() {
        return new OperableBigDecimal( unwrapped().negate() );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void incrementBy( OperableNumber<BigDecimal> increment ) {
        update( unwrapped().add( increment.unwrapped() ) );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableNumber<BigDecimal> one() {
        return ONE;
    }
}
