/*
 Copyright 2004-2008 Paul R. Holser, Jr.  All rights reserved.
 Licensed under the Academic Free License version 3.0
 */

package jaggregate.internal.number;

import java.math.BigInteger;

/**
 * Wrapper for a {@code BigInteger}.
 *
 * @author <a href="mailto:pholser@alumni.rice.edu">Paul Holser</a>
 * @version $Id: OperableBigInteger.java,v 1.1 2008/03/26 18:01:24 pholser Exp $
 */
public class OperableBigInteger extends OperableNumber<BigInteger> {
    private static final long serialVersionUID = -1L;

    private static final OperableBigInteger ONE =
        new OperableBigInteger( BigInteger.ONE ) {
            private static final long serialVersionUID = -1L;

            @Override
            public void incrementBy( OperableNumber<BigInteger> increment ) {
                throw new UnsupportedOperationException();
            }
        };

    /**
     * Creates a wrapper for the given number.
     *
     * @param wrapped the wrapped number
     */
    public OperableBigInteger( BigInteger wrapped ) {
        super( wrapped );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean isZero() {
        return unwrapped().equals( BigInteger.ZERO );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableBigInteger plus( BigInteger addend ) {
        return new OperableBigInteger( unwrapped().add( addend ) );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableBigInteger plus( int addend ) {
        return new OperableBigInteger( unwrapped().add( BigInteger.valueOf( addend ) ) );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableBigInteger minus( BigInteger subtrahend ) {
        return new OperableBigInteger( unwrapped().subtract( subtrahend ) );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableBigInteger times( BigInteger multiplier ) {
        return new OperableBigInteger( unwrapped().multiply( multiplier ) );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableBigInteger times( int multiplier ) {
        return new OperableBigInteger(
            unwrapped().multiply( BigInteger.valueOf( multiplier ) ) );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableBigInteger dividedBy( BigInteger divisor ) {
        return new OperableBigInteger( unwrapped().divide( divisor ) );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableBigInteger modulo( BigInteger divisor ) {
        return new OperableBigInteger( unwrapped().remainder( divisor ) );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean isPositive() {
        return unwrapped().signum() > 0;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean isNegative() {
        return unwrapped().signum() < 0;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean greaterThan( BigInteger comparand ) {
        return unwrapped().compareTo( comparand ) > 0;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean greaterThanOrEqualTo( BigInteger comparand ) {
        return unwrapped().compareTo( comparand ) >= 0;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean lessThan( BigInteger comparand ) {
        return unwrapped().compareTo( comparand ) < 0;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean lessThanOrEqualTo( BigInteger comparand ) {
        return unwrapped().compareTo( comparand ) <= 0;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableBigInteger negated() {
        return new OperableBigInteger( unwrapped().negate() );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void incrementBy( OperableNumber<BigInteger> increment ) {
        update( unwrapped().add( increment.unwrapped() ) );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableNumber<BigInteger> one() {
        return ONE;
    }
}
