/*
 Copyright 2004-2008 Paul R. Holser, Jr.  All rights reserved.
 Licensed under the Academic Free License version 3.0
 */

package jaggregate.internal.number;

/**
 * Wrapper for a {@code byte}.
 *
 * @author <a href="mailto:pholser@alumni.rice.edu">Paul Holser</a>
 * @version $Id: OperableByte.java,v 1.1 2008/03/26 18:01:29 pholser Exp $
 */
public class OperableByte extends OperableNumber<Byte> {
    private static final long serialVersionUID = -1L;

    private static final OperableByte ONE = new OperableByte( (byte) 1 ) {
        private static final long serialVersionUID = -1L;

        @Override
        public void incrementBy( OperableNumber<Byte> increment ) {
            throw new UnsupportedOperationException();
        }
    };

    /**
     * Creates a wrapper for the given number.
     *
     * @param wrapped the wrapped number
     */
    public OperableByte( Byte wrapped ) {
        super( wrapped );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean isZero() {
        return unwrapped() == 0;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableByte plus( Byte addend ) {
        return new OperableByte( (byte) ( unwrapped() + addend ) );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableByte plus( int addend ) {
        return new OperableByte( (byte) ( unwrapped() + addend ) );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableByte minus( Byte subtrahend ) {
        return new OperableByte( (byte) ( unwrapped() - subtrahend ) );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableByte times( Byte multiplier ) {
        return new OperableByte( (byte) ( unwrapped() * multiplier ) );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableByte times( int multiplier ) {
        return new OperableByte( (byte) ( unwrapped() * multiplier ) );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableByte dividedBy( Byte divisor ) {
        return new OperableByte( (byte) ( unwrapped() / divisor ) );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableByte modulo( Byte divisor ) {
        return new OperableByte( (byte) ( unwrapped() % divisor ) );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean isPositive() {
        return unwrapped() > 0;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean isNegative() {
        return unwrapped() < 0;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean greaterThan( Byte comparand ) {
        return unwrapped() > comparand;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean greaterThanOrEqualTo( Byte comparand ) {
        return unwrapped() >= comparand;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean lessThan( Byte comparand ) {
        return unwrapped() < comparand;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean lessThanOrEqualTo( Byte comparand ) {
        return unwrapped() <= comparand;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableByte negated() {
        return new OperableByte( (byte) -unwrapped() );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void incrementBy( OperableNumber<Byte> increment ) {
        update( (byte) ( unwrapped() + increment.unwrapped() ) );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableNumber<Byte> one() {
        return ONE;
    }
}
