/*
 Copyright 2004-2008 Paul R. Holser, Jr.  All rights reserved.
 Licensed under the Academic Free License version 3.0
 */

package jaggregate.internal.number;

/**
 * Wrapper for a {@code double}.
 *
 * @author <a href="mailto:pholser@alumni.rice.edu">Paul Holser</a>
 * @version $Id: OperableDouble.java,v 1.1 2008/03/26 18:01:29 pholser Exp $
 */
public class OperableDouble extends OperableNumber<Double> {
    private static final long serialVersionUID = -1L;

    private static final OperableDouble ONE = new OperableDouble( 1D ) {
        private static final long serialVersionUID = -1L;

        @Override
        public void incrementBy( OperableNumber<Double> increment ) {
            throw new UnsupportedOperationException();
        }
    };

    /**
     * Creates a wrapper for the given number.
     *
     * @param wrapped the wrapped number
     */
    public OperableDouble( Double wrapped ) {
        super( wrapped );
    }

    /**
     * Creates a wrapper for the given number.
     *
     * @param wrapped the wrapped number
     */
    public OperableDouble( Long wrapped ) {
        super( wrapped.doubleValue() );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean isZero() {
        return unwrapped() == 0D;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableDouble plus( Double addend ) {
        return new OperableDouble( unwrapped() + addend );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableDouble plus( int addend ) {
        return new OperableDouble( unwrapped() + addend );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableDouble minus( Double subtrahend ) {
        return new OperableDouble( unwrapped() - subtrahend );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableDouble times( Double multiplier ) {
        return new OperableDouble( unwrapped() * multiplier );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableDouble times( int multiplier ) {
        return new OperableDouble( unwrapped() * multiplier );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableDouble dividedBy( Double divisor ) {
        return new OperableDouble( (long) ( unwrapped() / divisor ) );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableDouble modulo( Double divisor ) {
        return new OperableDouble( unwrapped() % divisor );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean isPositive() {
        return unwrapped() > 0D;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean isNegative() {
        return unwrapped() < 0D;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean greaterThan( Double comparand ) {
        return unwrapped() > comparand;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean greaterThanOrEqualTo( Double comparand ) {
        return unwrapped() >= comparand;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean lessThan( Double comparand ) {
        return unwrapped() < comparand;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean lessThanOrEqualTo( Double comparand ) {
        return unwrapped() <= comparand;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableDouble negated() {
        return new OperableDouble( -unwrapped() );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void incrementBy( OperableNumber<Double> increment ) {
        update( unwrapped() + increment.unwrapped() );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableNumber<Double> one() {
        return ONE;
    }
}
