/*
 Copyright 2004-2008 Paul R. Holser, Jr.  All rights reserved.
 Licensed under the Academic Free License version 3.0
 */

package jaggregate.internal.number;

/**
 * Wrapper for a {@code float}.
 *
 * @author <a href="mailto:pholser@alumni.rice.edu">Paul Holser</a>
 * @version $Id: OperableFloat.java,v 1.1 2008/03/26 18:01:26 pholser Exp $
 */
public class OperableFloat extends OperableNumber<Float> {
    private static final long serialVersionUID = -1L;

    private static final OperableFloat ONE = new OperableFloat( 1F ) {
        private static final long serialVersionUID = -1L;

        @Override
        public void incrementBy( OperableNumber<Float> increment ) {
            throw new UnsupportedOperationException();
        }
    };

    /**
     * Creates a wrapper for the given number.
     *
     * @param wrapped the wrapped number
     */
    public OperableFloat( Float wrapped ) {
        super( wrapped );
    }

    /**
     * Creates a wrapper for the given number.
     *
     * @param wrapped the wrapped number
     */
    public OperableFloat( Integer wrapped ) {
        this( wrapped.floatValue() );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean isZero() {
        return unwrapped() == 0F;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableFloat plus( Float addend ) {
        return new OperableFloat( unwrapped() + addend );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableFloat plus( int addend ) {
        return new OperableFloat( unwrapped() + addend );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableFloat minus( Float subtrahend ) {
        return new OperableFloat( unwrapped() - subtrahend );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableFloat times( Float multiplier ) {
        return new OperableFloat( unwrapped() * multiplier );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableFloat times( int multiplier ) {
        return new OperableFloat( unwrapped() * multiplier );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableFloat dividedBy( Float divisor ) {
        return new OperableFloat( (int) ( unwrapped() / divisor ) );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableFloat modulo( Float divisor ) {
        return new OperableFloat( unwrapped() % divisor );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean isPositive() {
        return unwrapped() > 0F;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean isNegative() {
        return unwrapped() < 0F;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean greaterThan( Float comparand ) {
        return unwrapped() > comparand;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean greaterThanOrEqualTo( Float comparand ) {
        return unwrapped() >= comparand;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean lessThan( Float comparand ) {
        return unwrapped() < comparand;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean lessThanOrEqualTo( Float comparand ) {
        return unwrapped() <= comparand;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableFloat negated() {
        return new OperableFloat( -unwrapped() );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void incrementBy( OperableNumber<Float> increment ) {
        update( unwrapped() + increment.unwrapped() );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableNumber<Float> one() {
        return ONE;
    }
}
