/*
 Copyright 2004-2008 Paul R. Holser, Jr.  All rights reserved.
 Licensed under the Academic Free License version 3.0
 */

package jaggregate.internal.number;

/**
 * Wrapper for an {@code int}.
 *
 * @author <a href="mailto:pholser@alumni.rice.edu">Paul Holser</a>
 * @version $Id: OperableInteger.java,v 1.1 2008/03/26 18:01:26 pholser Exp $
 */
public class OperableInteger extends OperableNumber<Integer> {
    private static final long serialVersionUID = -1L;

    private static final OperableInteger ONE = new OperableInteger( 1 ) {
        private static final long serialVersionUID = -1L;

        @Override
        public void incrementBy( OperableNumber<Integer> increment ) {
            throw new UnsupportedOperationException();
        }
    };

    /**
     * Creates a wrapper for the given number.
     *
     * @param wrapped the wrapped number
     */
    public OperableInteger( Integer wrapped ) {
        super( wrapped );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean isZero() {
        return unwrapped() == 0;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableInteger plus( Integer addend ) {
        return new OperableInteger( unwrapped() + addend );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableInteger plus( int addend ) {
        return new OperableInteger( unwrapped() + addend );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableInteger minus( Integer subtrahend ) {
        return new OperableInteger( unwrapped() - subtrahend );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableInteger times( Integer multiplier ) {
        return new OperableInteger( unwrapped() * multiplier );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableInteger times( int multiplier ) {
        return new OperableInteger( unwrapped() * multiplier );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableInteger dividedBy( Integer divisor ) {
        return new OperableInteger( unwrapped() / divisor );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableInteger modulo( Integer divisor ) {
        return new OperableInteger( unwrapped() % divisor );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean isPositive() {
        return unwrapped() > 0;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean isNegative() {
        return unwrapped() < 0;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean greaterThan( Integer comparand ) {
        return unwrapped() > comparand;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean greaterThanOrEqualTo( Integer comparand ) {
        return unwrapped() >= comparand;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean lessThan( Integer comparand ) {
        return unwrapped() < comparand;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean lessThanOrEqualTo( Integer comparand ) {
        return unwrapped() <= comparand;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableInteger negated() {
        return new OperableInteger( -unwrapped() );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void incrementBy( OperableNumber<Integer> increment ) {
        update( unwrapped() + increment.unwrapped() );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableNumber<Integer> one() {
        return ONE;
    }
}
