/*
 Copyright 2004-2008 Paul R. Holser, Jr.  All rights reserved.
 Licensed under the Academic Free License version 3.0
 */

package jaggregate.internal.number;

/**
 * Wrapper for a {@code long}.
 *
 * @author <a href="mailto:pholser@alumni.rice.edu">Paul Holser</a>
 * @version $Id: OperableLong.java,v 1.1 2008/03/26 18:01:25 pholser Exp $
 */
public class OperableLong extends OperableNumber<Long> {
    private static final long serialVersionUID = -1L;

    private static final OperableLong ONE = new OperableLong( 1L ) {
        private static final long serialVersionUID = -1L;

        @Override
        public void incrementBy( OperableNumber<Long> increment ) {
            throw new UnsupportedOperationException();
        }
    };

    /**
     * Creates a wrapper for the given number.
     *
     * @param wrapped the wrapped number
     */
    public OperableLong( Long wrapped ) {
        super( wrapped );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean isZero() {
        return unwrapped() == 0L;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableLong plus( Long addend ) {
        return new OperableLong( unwrapped() + addend );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableLong plus( int addend ) {
        return new OperableLong( unwrapped() + addend );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableLong minus( Long subtrahend ) {
        return new OperableLong( unwrapped() - subtrahend );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableLong times( Long multiplier ) {
        return new OperableLong( unwrapped() * multiplier );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableLong times( int multiplier ) {
        return new OperableLong( unwrapped() * multiplier );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableLong dividedBy( Long divisor ) {
        return new OperableLong( unwrapped() / divisor );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableLong modulo( Long divisor ) {
        return new OperableLong( unwrapped() % divisor );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean isPositive() {
        return unwrapped() > 0L;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean isNegative() {
        return unwrapped() < 0L;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean greaterThan( Long comparand ) {
        return unwrapped() > comparand;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean greaterThanOrEqualTo( Long comparand ) {
        return unwrapped() >= comparand;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean lessThan( Long comparand ) {
        return unwrapped() < comparand;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean lessThanOrEqualTo( Long comparand ) {
        return unwrapped() <= comparand;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableLong negated() {
        return new OperableLong( -unwrapped() );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void incrementBy( OperableNumber<Long> increment ) {
        update( unwrapped() + increment.unwrapped() );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableNumber<Long> one() {
        return ONE;
    }
}
