/*
 Copyright 2004-2008 Paul R. Holser, Jr.  All rights reserved.
 Licensed under the Academic Free License version 3.0
 */

package jaggregate.internal.number;

import java.math.BigDecimal;
import java.math.BigInteger;

/**
 * Makes different kinds of {@link OperableNumber}s.
 *
 * @author <a href="mailto:pholser@alumni.rice.edu">Paul Holser</a>
 * @version $Id: OperableNumberFactory.java,v 1.2 2008/03/30 03:07:11 pholser Exp $
 */
public class OperableNumberFactory {
    /**
     * Discourages instantiation.
     *
     * @throws UnsupportedOperationException always
     */
    protected OperableNumberFactory() {
        throw new UnsupportedOperationException();
    }

    /**
     * Wraps a number.
     *
     * @param <E> type of the wrapped number
     * @param wrapped number to wrap
     * @return wrapped version of {@code wrapped}
     */
    public static <E extends Number> OperableNumber<?> wrap( E wrapped ) {
        Class<?> wrappedClass = wrapped.getClass();

        if ( Integer.class.equals( wrappedClass ) )
            return new OperableInteger( Integer.class.cast( wrapped ) );

        if ( Float.class.equals( wrappedClass ) )
            return new OperableFloat( Float.class.cast( wrapped ) );

        if ( Double.class.equals( wrappedClass ) )
            return new OperableDouble( Double.class.cast( wrapped ) );

        if ( Long.class.equals( wrappedClass ) )
            return new OperableLong( Long.class.cast( wrapped ) );

        if ( Short.class.equals( wrappedClass ) )
            return new OperableShort( Short.class.cast( wrapped ) );

        if ( Byte.class.equals( wrappedClass ) )
            return new OperableByte( Byte.class.cast( wrapped ) );

        if ( BigInteger.class.equals( wrappedClass ) )
            return new OperableBigInteger( BigInteger.class.cast( wrapped ) );

        if ( BigDecimal.class.equals( wrappedClass ) )
            return new OperableBigDecimal( BigDecimal.class.cast( wrapped ) );

        throw new AssertionError( "cannot wrap a " + wrappedClass.getName() );
    }
}
