/*
 Copyright 2004-2008 Paul R. Holser, Jr.  All rights reserved.
 Licensed under the Academic Free License version 3.0
 */

package jaggregate.internal.number;

/**
 * Wrapper for a {@code short}.
 *
 * @author <a href="mailto:pholser@alumni.rice.edu">Paul Holser</a>
 * @version $Id: OperableShort.java,v 1.1 2008/03/26 18:01:25 pholser Exp $
 */
public class OperableShort extends OperableNumber<Short> {
    private static final long serialVersionUID = -1L;

    private static final OperableShort ONE = new OperableShort( (short) 1 ) {
        private static final long serialVersionUID = -1L;

        @Override
        public void incrementBy( OperableNumber<Short> increment ) {
            throw new UnsupportedOperationException();
        }
    };

    /**
     * Creates a wrapper for the given number.
     *
     * @param wrapped the wrapped number
     */
    public OperableShort( Short wrapped ) {
        super( wrapped );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean isZero() {
        return unwrapped() == 0;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableShort plus( Short addend ) {
        return new OperableShort( (short) ( unwrapped() + addend ) );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableShort plus( int addend ) {
        return new OperableShort( (short) ( unwrapped() + addend ) );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableShort minus( Short subtrahend ) {
        return new OperableShort( (short) ( unwrapped() - subtrahend ) );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableShort times( Short multiplier ) {
        return new OperableShort( (short) ( unwrapped() * multiplier ) );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableShort times( int multiplier ) {
        return new OperableShort( (short) ( unwrapped() * multiplier ) );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableShort dividedBy( Short divisor ) {
        return new OperableShort( (short) ( unwrapped() / divisor ) );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableShort modulo( Short divisor ) {
        return new OperableShort( (short) ( unwrapped() % divisor ) );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean isPositive() {
        return unwrapped() > 0;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean isNegative() {
        return unwrapped() < 0;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean greaterThan( Short comparand ) {
        return unwrapped() > comparand;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean greaterThanOrEqualTo( Short comparand ) {
        return unwrapped() >= comparand;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean lessThan( Short comparand ) {
        return unwrapped() < comparand;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean lessThanOrEqualTo( Short comparand ) {
        return unwrapped() <= comparand;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableShort negated() {
        return new OperableShort( (short) -unwrapped() );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void incrementBy( OperableNumber<Short> increment ) {
        update( (short) ( unwrapped() + increment.unwrapped() ) );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public OperableNumber<Short> one() {
        return ONE;
    }
}
